using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace ScrollingText
{
	/// <summary>
	/// A Simple Scrolling "Car Stereo" looking display.
	/// </summary>
	public class ScrollText : System.Windows.Forms.UserControl
	{ 
		/// <summary> 
		/// Simple Scrolling Car Stereo display.
		/// </summary>
#region Global Variables

		private int width; //Used in multiple methods
		private bool[,] LEDArray = new bool[60,20]; //The main "Display" array of bool.
		private System.Drawing.SolidBrush IntColor = new SolidBrush(Color.Lime); //The internal variable for color set by the Color Property
		private bool[,] TextArray; //The long array consisting of the whole string from the property.
		private int scroll = 8; //Scrolling Speed - (See property ScrollSpeed)
		private string scrolltext = "conipto, C# programmer"; //Internal scrolltext variable set by property DisplayText
		private System.ComponentModel.Container components = null;//Dunno what this does, MS put it in.
		private System.Timers.Timer T = new System.Timers.Timer ();//Timer for the scrolling.
		private Font font = new Font("Courier",8);//internal font used.
		private Graphics Refgr; //To not create and dispose every cycle.
        
#endregion

#region Public Properties

		public new System.Drawing.Font Font
		{
			get
			{
				return font;
			}
			set
			{
				font = value;
				MakeStringArray(scrolltext);
			}
		}

        [Description("This is the speed at which the string will scroll."),Category("Behavior")]
		public int ScrollSpeed
		{
			get
			{
				return scroll;
			}
			set
			{
				if(value<11)
				{
					scroll = value;
					switch(value)
					{
						case 1:
							T.Interval = 1000;
							break;
						case 2:
							T.Interval = 900;
							break;
						case 3:
							T.Interval = 750;
							break;
						case 4:
							T.Interval = 600;
							break;
						case 5:
							T.Interval = 400;
							break;
						case 6:
							T.Interval = 200;
							break;
						case 7:
							T.Interval = 150;
							break;
						case 8:
							T.Interval = 50;
							break;
						case 9:
							T.Interval = 25;
							break;
						case 10:
							T.Interval = 1;
							break;
					}
				}
			}
		}

        
        [Description("This is the string to scroll."),Category("Appearance")]
		public string ScrollingText
		{
			get
			{
				return scrolltext;
			}
			set
			{
				scrolltext = value;
				MakeStringArray(value);
			}
		}

		public override System.Drawing.Color ForeColor
		{
			get
			{
				return IntColor.Color;
			}
			set
			{
				IntColor = new SolidBrush(value);
			}
		}
		
        
#endregion

#region Internal methods
		private void MakeStringArray(string str)
		{
			Graphics gr;
			Bitmap img = new Bitmap (60,20);
			gr = Graphics.FromImage(img);
			//The font and size can be changed, but if it's over about 12, there's problems.
			width = (int)gr.MeasureString(str,font).Width;
			//Here you add to width if you want space between the text.
			width += 15; //this adds 15 "LED blocks" between the string as it rolls around.
			if (width<60) width=60;  //Self Explanatory.
			img = new Bitmap(width,20); //Resize the bitmap incase it's too small to draw on.
			//Note for above - could not use Measurestring without initialize graphics object -
			//something to think about for later to avoid declaring the bitmap twice.
			gr = Graphics.FromImage (img);
			gr.DrawString (str,font,Brushes.Blue,0,0); //brushes color is irrelevant, just needs to be something
			//Originall, I had drawn the string with blue, and compared it to it, but it uses different A and B values
			//So I just said if != Null, Basically.
			TextArray = new bool[width, 20]; //ReInitialize the Array.
			for (int x=0;x<width;x++)
			{
				for(int y=0;y<20;y++)
				{
					if (img.GetPixel(x,y).ToArgb()  != 0 )
					{
						TextArray[x,y] = true;
					}
					else
					{
						TextArray[x,y] = false;
					}
				}
			}
			gr.Dispose();
		}


		private void ShiftArrayLeft()
		{
			bool[] TempArray = new bool[20];
			for (int x=0;x<20;x++)
			{
				TempArray[x] = TextArray[0,x];
			}
			for (int x=1;x<width;x++)
			{
				for(int y=0;y<20;y++)
				{
					TextArray[x-1,y] = TextArray[x,y];
				}
			}
			for (int x=0;x<20;x++)
			{
				TextArray[width-1,x] = TempArray[x];
			}
			//Now make the LEDArray match.
			for (int x=0;x<60;x++)
			{
				for(int y=0;y<20;y++)
				{
					LEDArray[x,y] = TextArray[x,y];
				}
			}
			RefreshDisplay();
		}
		private void T_Tick(object sender, System.Timers.ElapsedEventArgs e)
		{
			if (DesignMode != true ) ShiftArrayLeft(); //might implement ShiftarrayRight later if I get bored.
		}
		private Bitmap bitgr  = new Bitmap(300,200);
		private Graphics GrphBits;

		private void RefreshDisplay()
		{		
			GrphBits = Graphics.FromImage(bitgr);
			for(int x=0;x<60;x++)
			{
				for(int y=0;y<20;y++)
				{
					if (LEDArray[x,y]) 
					{
						GrphBits.FillRectangle(IntColor ,new Rectangle((5*x)+1,(5*y)+1,4,4));
					}
					else
					{
						GrphBits.FillRectangle(Brushes.DarkSlateGray,new Rectangle((5*x)+1,(5*y)+1,4,4));
					}
				}
			}
			Refgr.DrawImage(bitgr,0,0);

		}
		
		protected override void OnPaint(PaintEventArgs e) 
		{
			e.Graphics.FillRectangle(SystemBrushes.ControlDarkDark,new Rectangle(0,0,base.Width-1,base.Height-1));
			if(DesignMode)
			{
				for(int x=0;x<60;x++)
				{
					for(int y=0;y<20;y++)
					{
						if (LEDArray[x,y]) 
						{
							e.Graphics.FillRectangle(IntColor,new Rectangle((5*x)+1,(5*y)+1,4,4));
						}
						else
						{
							e.Graphics.FillRectangle(Brushes.DarkSlateGray,new Rectangle((5*x)+1,(5*y)+1,4,4));
						}
					}
				}
			}
			e.Graphics.DrawRectangle(Pens.DarkGray,0,0,300,100);
			e.Graphics.DrawRectangle(Pens.Gray,0,0,301,101);
		}
#endregion

		public ScrollText()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			MakeStringArray(ScrollingText);
			ScrollSpeed = 8;
			T.Elapsed+=new System.Timers.ElapsedEventHandler(T_Tick);
			T.Start();
			base.SetStyle(ControlStyles.DoubleBuffer, true);
			base.SetStyle(ControlStyles.UserPaint, true);
			base.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			base.SetStyle(ControlStyles.ResizeRedraw, true);
			base.UpdateStyles();
			Refgr = base.CreateGraphics ();

		// TODO: Add any initialization after the InitializeComponent call

	}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// ScrollText
			// 
			this.Name = "ScrollText";
			this.ForeColor = ForeColor; //System.Drawing.Color.Lime;
			this.Size = new System.Drawing.Size(301, 101);
			this.ScrollSpeed = ScrollSpeed;
            this.Text = ScrollingText;

		}
		#endregion
	}
}
